/*
 *  Penguineyes version 0.5, a linuxified version of Xeyes
 *  Copyright (C) 1998 Neil Howie
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <gtk/gtk.h>

#include "penguin_colour.xpm"

/* global variables */
static GdkPixmap *sourcePenguin;
static GtkWidget *mainWindow;

static gboolean button_down = FALSE;

static GdkGC *gc;
static GdkColor *colour = NULL;

/* change the current drawing colour, not really used yet */
static void setColour(int red, int blue, int green) {
    colour->red = red * (65535/255);
    colour->green = green * (65535/255);
    colour->blue = blue * (65535/255);

    colour->pixel = (gulong)(red*65535 + green*256 + blue);

    gdk_color_alloc(gtk_widget_get_colormap(mainWindow), colour);
    gdk_gc_set_foreground(gc, colour);
}

static void quit()
{
    gtk_main_quit();
}

#define NUM_EYES 2

static struct coord {
    gint x;
    gint y;
} mouse, mold, iold[NUM_EYES];

/* functions for dragging window */
static gint button_press(GtkWidget *w, GdkEventButton *event)
{
    if (event->button == 1) {
	gdk_window_get_pointer(event->window, &mouse.x, &mouse.y, NULL);
	button_down = TRUE;
    }
    return FALSE;
}

static gint button_release(GtkWidget *w, GdkEventButton *event)
{
    if (event->button == 1) {
	button_down = FALSE;
    }
    return FALSE;
}

static gint motion_notify(GtkWidget *w, GdkEventMotion *event)
{
    if (button_down) {
	int mx, my, wx, wy;

	gdk_window_get_pointer(event->window, &mx, &my, NULL);
	gdk_window_get_position(mainWindow->window, &wx, &wy);
	wx += (mx - mouse.x);
	wy += (my - mouse.y);
      /* move it */
	gtk_widget_set_uposition(mainWindow, wx, wy);
    }
    return FALSE;
}

static const struct eyeball {
    gint x;
    gint y;
    gint width;
    gint height;
    gint radius;
} ball[NUM_EYES] = {
    { 67, 32, 18, 20, 7},
    { 98, 32, 18, 20, 7}
};


struct {
    float dx;
    float dy;
    int cx;
    int cy;

    int aw;
    int ah;
    int sx;
    int sy;
    int ex;
    int ey;
} trans[NUM_EYES];

/* some calculations in advance so they aren't calculated each time */
static void set_trans(int i)
{
    trans[i].cx = ball[i].x + ball[i].width/2;
    trans[i].cy = ball[i].y + ball[i].height/2;

    if (ball[i].width > ball[i].height) {
	trans[i].dx = 1.0;
	trans[i].dy = (float)ball[i].width/(float)ball[i].height;
    } else {
	trans[i].dx = (float)ball[i].height/(float)ball[i].width;
	trans[i].dy = 1.0;
    }

    trans[i].sx = ball[i].x + ball[i].radius;
    trans[i].sy = ball[i].y + ball[i].radius;

    trans[i].aw = ball[i].width - (ball[i].radius);
    trans[i].ah = ball[i].height - (ball[i].radius);

    trans[i].ex = ball[i].x + trans[i].aw;
    trans[i].ey = ball[i].y + trans[i].ah;
}

static void draw_eye(gint i, struct coord m)
{
    struct coord b, dif, d;
    float angle;

    dif.x = trans[i].cx - m.x;
    dif.y = trans[i].cy - m.y;

    d.x = MIN(ABS(dif.x), trans[i].aw);
    d.y = MIN(ABS(dif.y), trans[i].ah);

    angle = atan2((float)dif.y,(float)dif.x);
    b.x = (gint)(d.x*cos(angle))/2;
    b.y = (gint)(d.y*sin(angle))/2;

    b.x = trans[i].cx - b.x;
    b.y = trans[i].cy - b.y;

  // cover up the old eyeball by whatever was underneath it
    gdk_draw_pixmap(mainWindow->window, gc, sourcePenguin,
	    iold[i].x - ball[i].radius, iold[i].y - ball[i].radius,
	    iold[i].x - ball[i].radius, iold[i].y - ball[i].radius,
	    ball[i].radius*2,
	    ball[i].radius*2);

    setColour(0,0,0);
    gdk_draw_arc(mainWindow->window, gc, TRUE,
	    b.x - ball[i].radius, b.y - ball[i].radius,
	    ball[i].radius*2, ball[i].radius*2, 0, 360*64);

    iold[i].x = b.x;
    iold[i].y = b.y;
}

static gint timeout_func()
{
    int i;
    struct coord m;

    gdk_window_get_pointer(mainWindow->window, &m.x, &m.y, NULL);
    if (button_down || (m.x == mold.x && m.y == mold.y))
	return TRUE;

    mold.x = m.x;
    mold.y = m.y;
    for (i = 0; i < NUM_EYES; i++) {
	draw_eye(i, m);
    }
    return TRUE;
}


static void penguinInit()
{
    GdkBitmap *mask;
    GtkStyle *style;
    gint width, height;
    gint i;

    mainWindow = gtk_window_new( GTK_WINDOW_POPUP );

    gtk_widget_set_events(mainWindow, GDK_BUTTON_PRESS_MASK
	    | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK);
 
    gtk_widget_realize (mainWindow);
    mold.x = 0;
    mold.y = 0;
    for (i = 0; i < NUM_EYES; i++) {
	iold[i].x = 0;
	iold[i].y = 0;
    }

    style = gtk_widget_get_default_style();
    gc = style->black_gc;

    sourcePenguin = gdk_pixmap_create_from_xpm_d(mainWindow->window, &mask,
	    &style->bg[GTK_STATE_NORMAL], penguin_colour_xpm);

    gdk_window_set_back_pixmap(mainWindow->window, sourcePenguin, 0);

    gdk_window_get_size((GdkWindow*)sourcePenguin, &width, &height);
    gtk_widget_set_usize(GTK_WIDGET(mainWindow), width, height);

    colour = (GdkColor*)malloc(sizeof(GdkColor));

    setColour(0,0,0);
    gtk_widget_shape_combine_mask( mainWindow, mask, 0, 0 );
   
    gtk_signal_connect (GTK_OBJECT(mainWindow), "motion_notify_event",
                        GTK_SIGNAL_FUNC (motion_notify), NULL);

    gtk_signal_connect (GTK_OBJECT (mainWindow), "button_press_event",
                        GTK_SIGNAL_FUNC (button_press), NULL);

    gtk_signal_connect (GTK_OBJECT (mainWindow), "button_release_event",
                        GTK_SIGNAL_FUNC (button_release), NULL);

    gtk_signal_connect (GTK_OBJECT (mainWindow), "delete_event",
	    		GTK_SIGNAL_FUNC (quit), NULL);

    gtk_timeout_add(50, timeout_func, NULL);

    gtk_widget_show( mainWindow );


    for (i = 0; i < NUM_EYES; i++)
	set_trans(i);
}

int main(int argc, char **argv)
{
    gtk_init(&argc, &argv);
    penguinInit();
    gtk_main();
    return 0;
}
